@@ -29,12 +29,12 @@
 
 /////////////////////////////////////////////////////////////////////
 
-#ifndef RCSC_ACTION_BODY_INTERCEPT_H
+#ifndef RCSC_ACTION_BODY_INTERCEPTH
 #define RCSC_ACTION_BODy_INTERCEPT_H
 
 #include <rcsc/player/soccer_action.h>
 #include <rcsc/player/intercept_table.h>
-#include <rcsc/param/player_type.h>
+#include <rcsc/common/player_type.h>
 #include <rcsc/geom/vector_2d.h>
 
 namespace rcsc {
@@ -48,15 +48,17 @@ class Body_Intercept
 private:
     //! if true, agent must save recovery parameter
     const bool M_save_recovery;
-
+    const Vector2D M_face_point;
 public:
     /*!
       \brief construct with all parameters
       \param save_recovery if true, agent must save recovery parameter
     */
     explicit
-    Body_Intercept( const bool save_recovery = true )
+    Body_Intercept( const bool save_recovery = true,
+                    const Vector2D & face_point = Vector2D::INVALIDATED )
         : M_save_recovery( save_recovery )
+        , M_face_point( face_point )
       { }
 
     /*!
@@ -82,30 +84,33 @@ private:
       \param table const pointer to the cached table
       \return interception info object
     */
-    InterceptInfo getBestPoint( const WorldModel & wm,
-                                const InterceptTable * table ) const;
+    InterceptInfo getBestIntercept( const WorldModel & wm,
+                                    const InterceptTable * table ) const;
+
+    InterceptInfo getBestIntercept_Test( const WorldModel & wm,
+                                         const InterceptTable * table ) const;
 
     /*!
       \brief try to perform ball wait action
       \param agent pointer to the agent itself
+      \param target_point intercept target ball point
       \param info interception info that is considered now
       \return true if action is performed
     */
     bool doWaitTurn( PlayerAgent * agent,
+                     const Vector2D & target_point,
                      const InterceptInfo & info );
 
     /*!
       \brief adjutment dash action. if possible try to perform turn action.
       \param agent pointer to agent itself
       \param target_point intercept target ball point
-      \param dash_power origianal intercept dash power parameter
-      \param cycle estimated interception cycle
+      \param info interception info that is considered now
       \return true if action is performed
     */
     bool doInertiaDash( PlayerAgent * agent,
                         const Vector2D & target_point,
-                        const double & dash_power,
-                        const int cycle );
+                        const InterceptInfo & info );
 
 };
 
