@@ -40,7 +40,7 @@
 
 #include <rcsc/player/logger.h>
 #include <rcsc/player/player_agent.h>
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/server_param.h>
 #include <rcsc/geom/circle_2d.h>
 #include <rcsc/geom/ray_2d.h>
 
@@ -141,16 +141,16 @@ Body_KickOneStep::get_max_possible_vel( const AngleDeg & target_angle,
                                         const double & krate,
                                         const Vector2D & ball_vel )
 {
-    // ball info may be future esitimation.
+    // ball info may be the estimated value..
 
-    const double enable_acc
+    const double max_accel
         = std::min( ServerParam::i().maxPower() * krate,
                     ServerParam::i().ballAccelMax() );
 
     // origin point is current ball pos
     Ray2D desired_ray( Vector2D( 0.0, 0.0 ), target_angle );
     // center point is next ball pos relative to current ball pos
-    Circle2D next_reachable_circle( ball_vel, enable_acc );
+    Circle2D next_reachable_circle( ball_vel, max_accel );
 
 
     // sol is ball vel (= current ball vel + accel)
@@ -160,7 +160,7 @@ Body_KickOneStep::get_max_possible_vel( const AngleDeg & target_angle,
     if ( num == 0 )
     {
         dlog.addText( Logger::KICK,
-                      "%s:%d: get_max_possible_vel. to %.1f No solution"
+                      "%s:%d: get_max_possible_vel: angle=%.1f. No solution"
                       ,__FILE__, __LINE__,
                       target_angle.degree() );
         return Vector2D( 0.0, 0.0 );
@@ -175,65 +175,77 @@ Body_KickOneStep::get_max_possible_vel( const AngleDeg & target_angle,
             {
                 // can adjust angle at least
                 sol1.setLength( ServerParam::i().ballSpeedMax() );
+
                 dlog.addText( Logger::KICK,
-                              "%s:%d: get_max_possible_vel to %.1f. adjust."
-                              " max-vel=(%.2f, %.2f) r=%.2f, th=%.1f"
+                              "%s:%d: get_max_possible_vel: angle=%.1f."
+                              " 1 solution  adjust."
                               ,__FILE__, __LINE__,
-                              target_angle.degree(),
-                              sol1.x, sol1.y, sol1.r(), sol1.th().degree() );
-                return sol1;
+                              target_angle.degree() );
             }
+            else
+            {
+                // failed
+                sol1.assign( 0.0, 0.0 );
 
-            // failed
-            dlog.addText( Logger::KICK,
-                          "%s:%d: get_max_possible_vel to %.1f. failed."
-                          " vel=(%.2f, %.2f)r=%.2f"
-                          ,__FILE__, __LINE__,
-                          target_angle.degree(),
-                          sol1.x, sol1.y, sol1.r() );
-
-            return Vector2D( 0.0, 0.0 );
-        }
-        else
-        {
-            // can reach
-            dlog.addText( Logger::KICK,
-                          "%s:%d: get_max_possible_vel to %.1f. can reach."
-                          " vel=(%.2f, %.2f)r=%.2f"
-                          ,__FILE__, __LINE__,
-                          target_angle.degree(),
-                          sol1.x, sol1.y, sol1.r() );
-            return sol1;
+                dlog.addText( Logger::KICK,
+                              "%s:%d: get_max_possible_vel: angle=%.1f."
+                              " 1 solution. failed."
+                              ,__FILE__, __LINE__,
+                              target_angle.degree() );
+            }
         }
+#ifdef DEBUG
+        dlog.addText( Logger::KICK,
+                      "one kick -----> angle=%.1f max_vel=(%.2f, %.2f)r=%.2f",
+                      target_angle.degree(),
+                      sol1.x, sol1.y, sol1.r() );
+#endif
+        return sol1;
     }
 
 
     // num == 2
 
-    double tmp1 = sol1.r();
-    double tmp2 = sol2.r();
-    if ( tmp1 < tmp2 )
+    double length1 = sol1.r();
+    double length2 = sol2.r();
+
+    if ( length1 < length2 )
     {
         std::swap( sol1, sol2 );
-        std::swap( tmp1, tmp2 );
+        std::swap( length1, length2 );
         dlog.addText( Logger::KICK,
-                      "%s:%d: get_max_possible_vel. swap"
-                      ,__FILE__, __LINE__ );
-    }
-    if ( tmp1 > ServerParam::i().ballSpeedMax()
-         && tmp2 < ServerParam::i().ballSpeedMax() )
-    {
-        sol1.setLength( ServerParam::i().ballSpeedMax() );
-        dlog.addText( Logger::KICK,
-                      "%s:%d: get_max_possible_vel. normalize"
+                      "%s:%d: get_max_possible_vel: swap"
                       ,__FILE__, __LINE__ );
     }
 
+    if ( length1 > ServerParam::i().ballSpeedMax() )
+    {
+        if ( length2 > ServerParam::i().ballSpeedMax() )
+        {
+            sol1.assign( 0.0, 0.0 );
+
+            dlog.addText( Logger::KICK,
+                          "%s:%d: get_max_possible_vel: angle=%.1f."
+                          " 2 solutions. but never reach"
+                          ,__FILE__, __LINE__,
+                          target_angle.degree() );
+        }
+        else
+        {
+            sol1.setLength( ServerParam::i().ballSpeedMax() );
+
+            dlog.addText( Logger::KICK,
+                          "%s:%d: get_max_possible_vel: angle=%.1f."
+                          " 2 solutions. adjust to ballSpeedMax"
+                          ,__FILE__, __LINE__,
+                          target_angle.degree() );
+        }
+    }
+
     dlog.addText( Logger::KICK,
-                  "%s:%d: get_max_possible_vel to %.1f. can reach(2). vel=(%f, %f)"
-                  ,__FILE__, __LINE__,
+                  "one kick: 2 solutions: angle=%.1f max_vel=(%.2f, %.2f)r=%.2f",
                   target_angle.degree(),
-                  sol1.x, sol1.y );
+                  sol1.x, sol1.y, sol1.r() );
 
     return sol1;
 }
