@@ -42,7 +42,7 @@
 #include "player_command.h"
 
 #include <rcsc/math_util.h>
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/server_param.h>
 
 namespace rcsc {
 
@@ -394,9 +394,9 @@ ActionEffector::makeCommand( std::ostream & to )
         {
             dlog.addText( Logger::SYSTEM,
                           "  WARNING. no body command." );
-            std::cerr << M_world.teamName() << ": "
-                      << M_world.self().unum()
-                      << ' ' << M_world.time()
+            std::cerr << M_world.teamName() << ' '
+                      << M_world.self().unum()<< ": "
+                      << M_world.time()
                       << "  WARNING. no body command." << std::endl;
             // register dummy command
             PlayerTurnCommand turn( 0 );
@@ -471,9 +471,10 @@ ActionEffector::setKick( const double & power,
     double command_power = power;
     if ( command_power > ServerParam::i().maxPower() + 0.01 )
     {
-        std::cerr << M_world.teamName() << ": "
-                  << M_world.self().unum()
-                  << "kick power is over max. com=" << command_power
+        std::cerr << M_world.teamName() << ' '
+                  << M_world.self().unum() << ": "
+                  << M_world.time()
+                  << " kick power is over max. com=" << command_power
                   << " > sparam=" << ServerParam::i().maxPower() << std::endl;
         dlog.addText( Logger::ACTION,
                       "setKick. power over. %.10f",
@@ -482,8 +483,9 @@ ActionEffector::setKick( const double & power,
     }
     if ( command_power < 0.0 )
     {
-        std::cerr << M_world.teamName() << ": "
-                  << M_world.self().unum()
+        std::cerr << M_world.teamName() << ' '
+                  << M_world.self().unum() << ": "
+                  << M_world.time()
                   << " kick power is negative " << command_power
                   << std::endl;
         dlog.addText( Logger::ACTION,
@@ -553,7 +555,9 @@ conserve_dash_power( const WorldModel & world,
                                       : power );
     if ( required_stamina < 0.0 )
     {
-        std::cerr << world.self().unum()
+        std::cerr << world.teamName() << ' '
+                  << world.self().unum() << ": "
+                  << world.time()
                   << " dash power should be positive now"
                   << std::endl;
         dlog.addText( Logger::ACTION,
@@ -622,8 +626,9 @@ ActionEffector::setDash( const double & power )
     {
         dlog.addText( Logger::ACTION,
                       "dash power is too high" );
-        std::cerr << M_world.teamName() << ": "
-                  << M_world.self().unum()
+        std::cerr << M_world.teamName() << ' '
+                  << M_world.self().unum() << ": "
+                  << M_world.time()
                   << " dash power is too high: " << power << std::endl;
         command_power = ServerParam::i().maxPower();
     }
@@ -631,8 +636,9 @@ ActionEffector::setDash( const double & power )
     {
         dlog.addText( Logger::ACTION,
                       "dash power is too low" );
-        std::cerr << M_world.teamName() << ": "
-                  << M_world.self().unum()
+        std::cerr << M_world.teamName() << ' '
+                  << M_world.self().unum() << ": "
+                  << M_world.time()
                   << " dash power is too low;: " << power << std::endl;
         command_power = ServerParam::i().minPower();
     }
@@ -791,8 +797,9 @@ ActionEffector::setMove( const double & x,
     if ( std::fabs( command_y ) > ServerParam::i().pitchHalfWidth()
          || std::fabs( command_x ) > ServerParam::i().pitchHalfLength() )
     {
-        std::cerr << M_world.teamName() << ": "
-                  << M_world.self().unum()
+        std::cerr << M_world.teamName() << ' '
+                  << M_world.self().unum() << ": "
+                  << M_world.time()
                   << " Must move to a place in the pitch ("
                   << command_x << ", " << command_y << ")" << std::endl;
         dlog.addText( Logger::ACTION,
@@ -811,8 +818,9 @@ ActionEffector::setMove( const double & x,
     if ( ServerParam::i().kickoffOffside()
          && command_x > 0.0 )
     {
-        std::cerr << M_world.teamName() << ": "
-                  << M_world.self().unum()
+        std::cerr << M_world.teamName() << ' '
+                  << M_world.self().unum() << ": "
+                  << M_world.time()
                   << " Must move to a place in our half ("
                   << command_x << ", " << command_y << ")" << std::endl;
         dlog.addText( Logger::ACTION,
@@ -828,8 +836,9 @@ ActionEffector::setMove( const double & x,
     {
         if ( command_x < -ServerParam::i().pitchHalfLength() + 1.0 )
         {
-            std::cerr << M_world.teamName() << ": "
-                      << M_world.self().unum()
+            std::cerr << M_world.teamName() << ' '
+                      << M_world.self().unum() << ": "
+                      << M_world.time()
                       << " Must move to a place within penalty area(1) ("
                       << command_x << ", " << command_y << ")" << std::endl;
             dlog.addText( Logger::ACTION,
@@ -839,8 +848,9 @@ ActionEffector::setMove( const double & x,
         }
         if ( command_x > - ServerParam::i().ourPenaltyAreaLine() - 1.0 )
         {
-            std::cerr << M_world.teamName() << ": "
-                      << M_world.self().unum()
+            std::cerr << M_world.teamName() << ' '
+                      << M_world.self().unum() << ": "
+                      << M_world.time()
                       << " Must move to a place within penalty area(2) ("
                       << command_x << ", " << command_y << ")"<< std::endl;
             dlog.addText( Logger::ACTION,
@@ -850,8 +860,9 @@ ActionEffector::setMove( const double & x,
         }
         if ( command_y > ServerParam::i().penaltyAreaHalfWidth() - 1.0 )
         {
-            std::cerr << M_world.teamName() << ": "
-                      << M_world.self().unum()
+            std::cerr << M_world.teamName() << ' '
+                      << M_world.self().unum() << ": "
+                      << M_world.time()
                       << " Must move to a place within penalty area(3) ("
                       << command_x << ", " << command_y << ")"<< std::endl;
             dlog.addText( Logger::ACTION,
@@ -861,8 +872,9 @@ ActionEffector::setMove( const double & x,
         }
         if ( command_y < - ServerParam::i().penaltyAreaHalfWidth() + 1.0 )
         {
-            std::cerr << M_world.teamName() << ": "
-                      << M_world.self().unum()
+            std::cerr << M_world.teamName() << ' '
+                      << M_world.self().unum() << ": "
+                      << M_world.time()
                       << " Must move to a place within penalty area(4) ("
                       << command_x << ", " << command_y << ")"<< std::endl;
             dlog.addText( Logger::ACTION,
@@ -934,8 +946,9 @@ ActionEffector::setTackle( const double & power )
 
     if ( command_power > ServerParam::i().maxPower() + 0.01 )
     {
-        std::cerr << M_world.teamName() << ": "
-                  << M_world.self().unum()
+        std::cerr << M_world.teamName() << ' '
+                  << M_world.self().unum() << ": "
+                  << M_world.time()
                   << "tackle power overflow. com=" << command_power
                   << " > sparam=" << ServerParam::i().maxPower() << std::endl;
         dlog.addText( Logger::ACTION,
@@ -945,8 +958,9 @@ ActionEffector::setTackle( const double & power )
     }
     if ( command_power < ServerParam::i().minPower() - 0.01 )
     {
-        std::cerr << M_world.teamName() << ": "
-                  << M_world.self().unum()
+        std::cerr << M_world.teamName() << ' '
+                  << M_world.self().unum() << ": "
+                  << M_world.time()
                   << "tackle power underflow " << command_power << std::endl;
         dlog.addText( Logger::ACTION,
                       "Effector::setKick. power underflow. %.1f",
@@ -988,8 +1002,9 @@ ActionEffector::setTurnNeck( const AngleDeg & moment )
     // adjust for neck moment range
     if ( command_moment > ServerParam::i().maxNeckMoment() + 0.01 )
     {
-        std::cerr << M_world.teamName() << ": "
-                  << M_world.self().unum()
+        std::cerr << M_world.teamName() << ' '
+                  << M_world.self().unum() << ": "
+                  << M_world.time()
                   << "  doTurnNeck. over max moment. "
                   << command_moment << std::endl;
         dlog.addText( Logger::ACTION,
@@ -999,8 +1014,9 @@ ActionEffector::setTurnNeck( const AngleDeg & moment )
     }
     if ( command_moment < ServerParam::i().minNeckMoment() - 0.01 )
     {
-        std::cerr << M_world.teamName() << ": "
-                  << M_world.self().unum()
+        std::cerr << M_world.teamName() << ' '
+                  << M_world.self().unum() << ": "
+                  << M_world.time()
                   << "  doTurnNeck. under min moment. "
                   << command_moment << std::endl;
         dlog.addText( Logger::ACTION,
@@ -1241,6 +1257,24 @@ ActionEffector::queuedNextMyPos() const
     return M_world.self().pos() + vel;
 }
 
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+bool
+ActionEffector::queuedNextBallKickable() const
+{
+    if ( M_world.ball().rposCount() >= 3 )
+    {
+        return false;
+    }
+
+    Vector2D my_next = queuedNextMyPos();
+    Vector2D ball_next = queuedNextBallPos();
+
+    return my_next.dist( ball_next ) < M_world.self().kickableArea() - 0.06;
+}
+
 /*-------------------------------------------------------------------*/
 /*!
   estimate next ball pos
