@@ -497,6 +497,12 @@ public:
     */
     Vector2D queuedNextMyPos() const;
 
+    /*!
+      \brief check if ball will be kickable in the next cycle
+      \return true if ball will be kickable
+     */
+    bool queuedNextBallKickable() const;
+
     /*!
       \brief get estimated next ball position using queued action effect
       \return estimated ball position
