@@ -271,7 +271,7 @@ BodySensor::parse1( const char * msg,
     if ( *msg == 'n' ) // "none"
     {
         M_attentionto_side = NEUTRAL;
-        M_attentionto_unum = -1;
+        M_attentionto_unum = Unum_Unknown;
     }
     else if ( *msg == 'l' )
     {
@@ -354,7 +354,7 @@ BodySensor::parse2( const char * msg,
         if ( attentionto_target[0] == 'n' )
         {
             M_attentionto_side = NEUTRAL;
-            M_attentionto_unum = -1;;
+            M_attentionto_unum = Unum_Unknown;
         }
         else if ( attentionto_target[0] == 'l' )
         {
@@ -373,7 +373,7 @@ BodySensor::parse2( const char * msg,
             std::cerr << "Received unknown attentionto target ["
                       << attentionto_target << "]" << std::endl;
             M_attentionto_side = NEUTRAL;
-            M_attentionto_unum = -1;
+            M_attentionto_unum = Unum_Unknown;
         }
     }
     else if ( version >= 7.0
