@@ -181,6 +181,14 @@ public:
     */
     void update();
 
+    /*!
+      \brief set teammate intercept info mainly by heard info
+      \param unum uniform number
+      \param cycle interception cycle
+     */
+    void updateTeammate( const int unum,
+                         const int cycle );
+
     /*!
       \brief get minimal ball gettable cycle for self without stamina exaust
       \return cycle value to get the ball
