@@ -69,7 +69,7 @@ public:
           \brief init member variables by error value
         */
         PlayerT()
-            : unum_( -1 )
+            : unum_( Unum_Unknown )
             , goalie_( false )
             , pos_( Vector2D::INVALIDATED )
             , rpos_( Vector2D::INVALIDATED )
@@ -89,7 +89,7 @@ public:
           {
               pos_.invalidate();
               rpos_.invalidate();
-              unum_ = -1;
+              unum_ = Unum_Unknown;
               has_face_ = pointto_ = tackle_ = false;
           }
 
@@ -151,17 +151,13 @@ public:
 public:
     /*!
       \brief localize self.
-      \param lines seen line container
-      \param markers seen marker container
-      \param behind_markers seen behind marker container
+      \param see analyzed see info
       \param self_face pointer to the variable to store the localized face angle
       \param self_face_err pointer to the variable to store the localized face angle error
       \param self_pos pointer to the variable to store the localized self position
       \param self_pos_err pointer to the variable to store the localized self position error
     */
-    void localizeSelf( const VisualSensor::LineCont & lines,
-                       const VisualSensor::MarkerCont & markers,
-                       const VisualSensor::MarkerCont & behind_markers,
+    void localizeSelf( const VisualSensor & see,
                        double * self_face,
                        double * self_face_err,
                        Vector2D * self_pos,
@@ -169,7 +165,7 @@ public:
 
     /*!
       \brief localze ball relative info
-      \param balls seen ball container
+      \param see analyzed see info
       \param self_face localized self face angle
       \param self_face_err localized self face angle error
       \param rpos pointer to the variable to store the localized relative position
@@ -177,7 +173,7 @@ public:
       \param rvel pointer to the variable to store the localized relative velocity
       \param rvel_err pointer to the variable to store the localized relative velocity error
     */
-    void localizeBallRelative( const VisualSensor::BallCont & balls,
+    void localizeBallRelative( const VisualSensor & see,
                                const double & self_face,
                                const double & self_face_err,
                                Vector2D * rpos,
