@@ -172,6 +172,12 @@ public:
           return ( M_fout != NULL );
       }
 
+    /*!
+      \brief put message to file directry. no flush.
+      \param msg message
+     */
+    void print( const char * msg );
+
     /*!
       \brief add free message to buffer with cycle, level & message tag 'T'
       \param id debug flag id
