@@ -46,12 +46,13 @@
 #include "soccer_action.h"
 #include "soccer_intention.h"
 
+#include <rcsc/common/basic_client.h>
+#include <rcsc/common/player_param.h>
+#include <rcsc/common/player_type.h>
+#include <rcsc/common/server_param.h>
 #include <rcsc/param/param_map.h>
 #include <rcsc/param/cmd_line_parser.h>
 #include <rcsc/param/conf_file_parser.h>
-#include <rcsc/param/player_param.h>
-#include <rcsc/param/player_type.h>
-#include <rcsc/param/server_param.h>
 #include <rcsc/math_util.h>
 #include <rcsc/game_time.h>
 #include <rcsc/game_mode.h>
@@ -368,7 +369,7 @@ PlayerAgentImpl::isDecisionTiming( const long & msec_from_sense ) const
 
 */
 PlayerAgent::PlayerAgent()
-    : BasicClient()
+    : SoccerAgent()
     , M_impl( new PlayerAgentImpl( *this ) )
     , M_debug_client()
     , M_worldmodel()
@@ -468,7 +469,7 @@ PlayerAgent::seeTimeStamp() const
 
 */
 bool
-PlayerAgent::doInit( CmdLineParser & cmd_parser )
+PlayerAgent::initImpl( CmdLineParser & cmd_parser )
 {
     bool help = false;
     std::string player_config_file;
@@ -488,7 +489,7 @@ PlayerAgent::doInit( CmdLineParser & cmd_parser )
     {
         system_param_map.printHelp( std::cout );
         player_param_map.printHelp( std::cout );
-        setServerAlive( false );
+        M_client->setServerAlive( false );
         return false;
     }
 
@@ -529,95 +530,39 @@ PlayerAgent::doInit( CmdLineParser & cmd_parser )
 /*!
 
 */
-void
-PlayerAgent::doConnect()
+bool
+PlayerAgent::handleStart()
 {
+    if ( ! M_client )
+    {
+        return false;
+    }
+
     if ( config().host().empty() )
     {
         std::cerr << config().teamName()
                   << ": ***ERROR*** server host name is empty" << std::endl;
-        return;
+        M_client->setServerAlive( false );
+        return false;
     }
 
     // just create a connection. init command is automaticcaly sent
     // by BasicClient's run() method.
-    if ( ! connectTo( config().host().c_str(),
-                      config().port(),
-                      static_cast< long >( config().intervalMSec() ) ) )
+    if ( ! M_client->connectTo( config().host().c_str(),
+                                config().port(),
+                                static_cast< long >( config().intervalMSec() ) ) )
     {
         std::cerr << config().teamName()
                   << ": ***ERROR*** Failed to connect to ["
                   << config().host()
                   << "]"
                   << std::endl;
-        setServerAlive( false );
-    }
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
-*/
-void
-PlayerAgent::sendInitCommand()
-{
-    if ( ! isServerAlive() )
-    {
-        std::cerr << config().teamName()
-                  << ": server is not alive." << std::endl;
-        return;
+        M_client->setServerAlive( false );
+        return false;
     }
 
-    std::ostringstream ostrm;
-
-    // make command string
-
-    if ( config().reconnectNumber() < 1
-         || 11 < config().reconnectNumber() )
-    {
-        // normal case
-        PlayerInitCommand com( config().teamName(),
-                               config().version(),
-                               config().goalie() );
-        com.toStr( ostrm );
-    }
-    else
-    {
-        std::cout << config().teamName()
-                  << ": reconnect. number = "
-                  << config().reconnectNumber() << std::endl;
-        // reconnect
-        PlayerReconnectCommand com( config().teamName(),
-                                    config().reconnectNumber() );
-        com.toStr( ostrm );
-
-    }
-
-
-    // send to server
-
-    if ( sendMessage( ostrm.str().c_str() ) <= 0 )
-    {
-        std::cout << config().teamName()
-                  << ": Failed to init ...\nExit ..." << std::endl;
-        setServerAlive( false );
-    }
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
-*/
-void
-PlayerAgent::sendByeCommand()
-{
-    PlayerByeCommand com;
-    std::ostringstream ostrm;
-
-    com.toStr( ostrm );
-    sendMessage( ostrm.str().c_str() );
-
-    setServerAlive( false );
+    sendInitCommand();
+    return true;
 }
 
 /*-------------------------------------------------------------------*/
@@ -627,14 +572,21 @@ PlayerAgent::sendByeCommand()
 void
 PlayerAgent::handleMessage()
 {
+    if ( ! M_client )
+    {
+        std::cerr << "PlayerAgent::handleMessage(). Client is not registered."
+                  << std::endl;
+        return;
+    }
+
     int counter = 0;
     GameTime start_time = M_impl->current_time_;
 
     // receive and analyze message
-    while ( recvMessage() > 0 )
+    while ( M_client->recvMessage() > 0 )
     {
         ++counter;
-        parse( recvBuf() );
+        parse( M_client->message() );
     }
 
     // game cycle is change while several message parsing
@@ -685,8 +637,16 @@ PlayerAgent::handleMessage()
 
 */
 void
-PlayerAgent::handleTimeout()
+PlayerAgent::handleTimeout( const int timeout_count,
+                            const int waited_msec )
 {
+    if ( ! M_client )
+    {
+        std::cerr << "PlayerAgent::handleTimeout(). Client is not registered."
+                  << std::endl;
+        return;
+    }
+
     TimeStamp cur_time;
     cur_time.setCurrent();
     long msec_from_sense = -1;
@@ -706,17 +666,17 @@ PlayerAgent::handleTimeout()
                   "----- Timeout. msec from sense_body = [%ld] ms."
                   " Timeout count = %d",
                   msec_from_sense / ServerParam::i().slowDownFactor(),
-                  timeoutCount() );
+                  timeout_count );
 
     // estimate server down
-    if ( waitedMSec() > config().serverWaitSeconds() * 1000 )
+    if ( waited_msec > config().serverWaitSeconds() * 1000 )
     {
         std::cout << world().teamName() << ' '
                   << world().self().unum() << ": "
                   << "waited "
-                  << waitedMSec() / 1000
+                  << waited_msec / 1000
                   << " seconds. server down??" << std::endl;
-        setServerAlive( false );
+        M_client->setServerAlive( false );
         return;
     }
 
@@ -751,7 +711,7 @@ PlayerAgent::handleExit()
 void
 PlayerAgent::finalize()
 {
-    if ( isServerAlive() )
+    if ( M_client->isServerAlive() )
     {
         sendByeCommand();
     }
@@ -775,6 +735,63 @@ PlayerAgent::finalize()
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+PlayerAgent::sendInitCommand()
+{
+    std::ostringstream ostrm;
+
+    // make command string
+
+    if ( config().reconnectNumber() < 1
+         || 11 < config().reconnectNumber() )
+    {
+        // normal case
+        PlayerInitCommand com( config().teamName(),
+                               config().version(),
+                               config().goalie() );
+        com.toStr( ostrm );
+    }
+    else
+    {
+        std::cout << config().teamName()
+                  << ": reconnect. number = "
+                  << config().reconnectNumber() << std::endl;
+        // reconnect
+        PlayerReconnectCommand com( config().teamName(),
+                                    config().reconnectNumber() );
+        com.toStr( ostrm );
+
+    }
+
+    // send to server
+    if ( M_client->sendMessage( ostrm.str().c_str() ) <= 0 )
+    {
+        std::cout << config().teamName()
+                  << ": Failed to init ...\nExit ..." << std::endl;
+        M_client->setServerAlive( false );
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+PlayerAgent::sendByeCommand()
+{
+    PlayerByeCommand com;
+    std::ostringstream ostrm;
+
+    com.toStr( ostrm );
+    M_client->sendMessage( ostrm.str().c_str() );
+
+    M_client->setServerAlive( false );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 PlayerAgent::setDebugFlags()
@@ -1208,41 +1225,53 @@ PlayerAgent::analyzeHearPlayer(  const char * msg )
     // set new info to world model
     if ( M_impl->audio_.playerMessage().time_ == M_impl->current_time_ )
     {
-        switch ( M_impl->audio_.playerMessage().type_ ) {
-        case PlayerMessage::BALL_INFO:
-            M_worldmodel.setHeardBallInfo( M_impl->audio_.ballPos(),
-                                           M_impl->audio_.ballVel(),
-                                           M_impl->current_time_ );
-            break;
-        case PlayerMessage::PASS_INFO:
-            M_worldmodel.setHeardPassInfo( M_impl->audio_.receiverNumber(),
-                                           M_impl->audio_.receivePos(),
-                                           M_impl->current_time_ );
-            break;
-        case PlayerMessage::OPPONENT_GOALIE_INFO:
-            M_worldmodel.setHeardGoalieInfo( M_impl->audio_.goalieNumber(),
-                                             M_impl->audio_.goaliePos(),
-                                             M_impl->current_time_ );
-            break;
-        case PlayerMessage::OFFSIDE_LINE_INFO:
-            M_worldmodel.setHeardOffsideLineInfo( M_impl->audio_.offsideLineX(),
-                                                  M_impl->current_time_ );
-            break;
-        case PlayerMessage::DEFENSE_LINE_INFO:
-            M_worldmodel.setHeardDefenseLineInfo( M_impl->audio_.defenseLineX(),
-                                                  M_impl->current_time_ );
-            break;
-        case PlayerMessage::WAIT_REQUEST:
-            M_worldmodel.setHeardWaitRequest( M_impl->current_time_ );
-            break;
-        case PlayerMessage::ATTACK_REQUEST:
-            M_worldmodel.setHeardAttackRequest( M_impl->current_time_ );
-            break;
-        case PlayerMessage::INTERCEPT:
-        case PlayerMessage::HEY_PASS:
-        case PlayerMessage::ILLEGAL:
-        default:
-            break;
+        const std::vector< PlayerMessage::Info >::const_iterator end
+            = M_impl->audio_.playerMessage().info_.end();
+        for ( std::vector< PlayerMessage::Info >::const_iterator it
+                  = M_impl->audio_.playerMessage().info_.begin();
+              it != end;
+              ++it )
+        {
+            switch ( it->type_ ) {
+            case PlayerMessage::BALL_INFO:
+                M_worldmodel.setHeardBallInfo( M_impl->audio_.ballPos(),
+                                               M_impl->audio_.ballVel(),
+                                               M_impl->current_time_ );
+                break;
+            case PlayerMessage::PASS_INFO:
+                M_worldmodel.setHeardPassInfo( M_impl->audio_.receiverNumber(),
+                                               M_impl->audio_.receivePos(),
+                                               M_impl->current_time_ );
+                break;
+            case PlayerMessage::OPPONENT_GOALIE_INFO:
+                M_worldmodel.setHeardGoalieInfo( M_impl->audio_.goalieNumber(),
+                                                 M_impl->audio_.goaliePos(),
+                                                 M_impl->current_time_ );
+                break;
+            case PlayerMessage::OFFSIDE_LINE_INFO:
+                M_worldmodel.setHeardOffsideLineInfo( M_impl->audio_.offsideLineX(),
+                                                      M_impl->current_time_ );
+                break;
+            case PlayerMessage::DEFENSE_LINE_INFO:
+                M_worldmodel.setHeardDefenseLineInfo( M_impl->audio_.defenseLineX(),
+                                                      M_impl->current_time_ );
+                break;
+            case PlayerMessage::WAIT_REQUEST:
+                M_worldmodel.setHeardWaitRequest( M_impl->current_time_ );
+                break;
+            case PlayerMessage::ATTACK_REQUEST:
+                M_worldmodel.setHeardAttackRequest( M_impl->current_time_ );
+                break;
+            case PlayerMessage::INTERCEPT_INFO:
+                M_worldmodel.setHeardInterceptInfo( M_impl->audio_.interceptNumber(),
+                                                    M_impl->audio_.interceptCycle(),
+                                                    M_impl->current_time_ );
+                break;
+            case PlayerMessage::HEY_PASS:
+            case PlayerMessage::UNKNOWN:
+            default:
+                break;
+            }
         }
     }
 }
@@ -1364,15 +1393,15 @@ PlayerAgent::analyzeServerParam( const char * msg )
     if ( ! ServerParam::i().synchMode()
          && ServerParam::i().slowDownFactor() != 1 )
     {
-        long interval = config().intervalMSec();
-        interval *= ServerParam::i().slowDownFactor();
+        long interval = ( config().intervalMSec()
+                          * ServerParam::i().slowDownFactor() );
         /*
           std::cout << "slow_down_factor is changed. new simst="
           << ServerParam::i().simulatorStep()
           << "  new interval=" << interval
           << std::endl;
         */
-        setIntervalMSec( interval );
+        M_client->setIntervalMSec( interval );
     }
 }
 
@@ -1414,7 +1443,7 @@ PlayerAgent::analyzeInit( const char * msg )
     {
         std::cout << config().teamName()
                   << ": failed to get an init message: " << msg << std::endl;
-        setServerAlive( false );
+        M_client->setServerAlive( false );
         return;
     }
 
@@ -1426,7 +1455,7 @@ PlayerAgent::analyzeInit( const char * msg )
         std::cerr << config().teamName()
                   << ": parsed reconnect, but reconect number is not specified??"
                   << std::endl;
-        setServerAlive( false );
+        M_client->setServerAlive( false );
         return;
     }
 
@@ -1459,7 +1488,7 @@ PlayerAgent::analyzeInit( const char * msg )
                                       sidetype, myunum,
                                       config().goalie() ) )
     {
-        setServerAlive( false );
+        M_client->setServerAlive( false );
         return;
     }
 
@@ -1474,9 +1503,7 @@ PlayerAgent::analyzeInit( const char * msg )
 
     if ( config().debugWrite() )
     {
-        M_debug_client.open( config().teamName(),
-                             myunum,
-                             config().logDir() );
+        M_debug_client.setWriteMode( true );
     }
 
     if ( config().debug() )
@@ -1495,7 +1522,8 @@ PlayerAgent::analyzeInit( const char * msg )
                 ostrm << '/';
             }
         }
-        ostrm << config().teamName() << "-" << myunum << ".log";
+        ostrm << config().teamName() << "-" << myunum
+              << config().logExt();
         std::string filepath = ostrm.str();
         //std::cerr << unum() << " Debug File is [" << filepath << "]" << std::endl;
         dlog.open( filepath.c_str() );
@@ -1547,14 +1575,10 @@ PlayerAgent::analyzeInit( const char * msg )
         dlog.addText( Logger::SYSTEM,
                       "---- send[%s]",
                       ostrm.str().c_str() );
-        sendMessage( ostrm.str().c_str() );
+        M_client->sendMessage( ostrm.str().c_str() );
         //////////////////////////
     }
 
-    ////////////////////////////////////////////////////////
-
-    // this method is pure virtual
-    initTeam();
 }
 
 /*-------------------------------------------------------------------*/
@@ -1567,7 +1591,8 @@ PlayerAgent::analyzeChangePlayerType( const char * msg )
     // teammate: "(change_player_type <unum> <type>)\n"
     // opponent: "(change_player_type <unum>)\n"
 
-    int unum = -1, type = -1;
+    int unum = Unum_Unknown;
+    int type = Hetero_Unknown;
 
     dlog.addText( Logger::SENSOR,
                   "parse change_player_type [%s]",
@@ -1604,7 +1629,9 @@ PlayerAgent::analyzeOK( const char * msg )
                   "===receive ok [%s]",
                   msg );
 
-    if ( ! std::strncmp( msg, "(ok compression ", std::strlen( "(ok compression " ) ) )
+    if ( ! std::strncmp( msg,
+                         "(ok compression ",
+                         std::strlen( "(ok compression " ) ) )
     {
         int level = 0;
         if ( std::sscanf( msg, "(ok compression %d)", &level ) == 1 )
@@ -1614,11 +1641,13 @@ PlayerAgent::analyzeOK( const char * msg )
                       << world().time()
                       << " set compression level " << level
                       << std::endl;
-            // setCompressionLevel( level );
+            M_client->setCompressionLevel( level );
             return;
         }
     }
-    else if ( ! std::strncmp( msg, "(ok clang ", std::strlen( "(ok clang " ) ) )
+    else if ( ! std::strncmp( msg,
+                              "(ok clang ",
+                              std::strlen( "(ok clang " ) ) )
     {
         // (ok clang (ver 7 8))
         int vermin, vermax;
@@ -1712,7 +1741,7 @@ PlayerAgent::action()
     // ------------------------------------------------------------------------
     // last update
     // update positining matrix, offside line, defense line, etc.
-    M_worldmodel.updateJustBeforeDecision( M_effector,
+    M_worldmodel.updateJustBeforeDecision( effector(),
                                            M_impl->current_time_ );
     // reset last action effect
     M_effector.reset();
@@ -1726,14 +1755,14 @@ PlayerAgent::action()
         adjustSeeSynchSynchMode();
     }
 
-    doAction(); // this is pure virtual method
+    actionImpl(); // this is pure virtual method
     doArmAction();
     doViewAction();
     doNeckAction();
-    doCommunication();
+    communicationImpl();
 
     // ------------------------------------------------------------------------
-    // set command effect
+    // set command effect. these must be called before command composing.
     // set self view mode, pointto and attentionto info.
     M_worldmodel.setCommandEffect( effector() );
     // set cycles till next see, update estimated next see arrival timing
@@ -1751,7 +1780,7 @@ PlayerAgent::action()
             dlog.addText( Logger::SYSTEM,
                           "---- send[%s]",
                           str.c_str() );
-            sendMessage( str.c_str() );
+            M_client->sendMessage( str.c_str() );
         }
     }
 
@@ -1846,7 +1875,8 @@ PlayerAgent::adjustSeeSynchNormalMode()
         PlayerChangeViewCommand com( ViewWidth::NORMAL, ViewQuality::HIGH );
         std::ostringstream ostrm;
         com.toStr( ostrm );
-        sendMessage( ostrm.str().c_str() );
+        M_client->sendMessage( ostrm.str().c_str() );
+
         dlog.addText( Logger::SYSTEM,
                       "---- send[%s] see sync",
                       ostrm.str().c_str() );
@@ -1873,7 +1903,7 @@ PlayerAgent::adjustSeeSynchNormalMode()
             PlayerChangeViewCommand com( ViewWidth::NARROW, ViewQuality::HIGH );
             std::ostringstream ostrm;
             com.toStr( ostrm );
-            sendMessage( ostrm.str().c_str() );
+            M_client->sendMessage( ostrm.str().c_str() );
 
             dlog.addText( Logger::SYSTEM,
                           "---- send[%s] no sync. change to high",
@@ -1894,7 +1924,7 @@ PlayerAgent::adjustSeeSynchNormalMode()
         PlayerChangeViewCommand com( ViewWidth::NARROW, ViewQuality::LOW );
         std::ostringstream ostrm;
         com.toStr( ostrm );
-        sendMessage( ostrm.str().c_str() );
+        M_client->sendMessage( ostrm.str().c_str() );
 
         dlog.addText( Logger::SYSTEM,
                       "---- send[%s] prepare see sync",
@@ -1933,7 +1963,7 @@ PlayerAgent::adjustSeeSynchSynchMode()
         PlayerChangeViewCommand com( ViewWidth::NARROW, ViewQuality::HIGH );
         std::ostringstream ostrm;
         com.toStr( ostrm );
-        sendMessage( ostrm.str().c_str() );
+        M_client->sendMessage( ostrm.str().c_str() );
 
         dlog.addText( Logger::SYSTEM,
                       "---- send[%s] synch_mode. see synch",
@@ -1961,7 +1991,7 @@ PlayerAgent::adjustSeeSynchSynchMode()
             PlayerChangeViewCommand com( ViewWidth::NARROW, ViewQuality::HIGH );
             std::ostringstream ostrm;
             com.toStr( ostrm );
-            sendMessage( ostrm.str().c_str() );
+            M_client->sendMessage( ostrm.str().c_str() );
 
             dlog.addText( Logger::SYSTEM,
                           "---- send[%s] synch_mode. no sync. change to high",
@@ -1982,7 +2012,7 @@ PlayerAgent::adjustSeeSynchSynchMode()
         PlayerChangeViewCommand com( ViewWidth::NARROW, ViewQuality::LOW );
         std::ostringstream ostrm;
         com.toStr( ostrm );
-        sendMessage( ostrm.str().c_str() );
+        M_client->sendMessage( ostrm.str().c_str() );
 
         dlog.addText( Logger::SYSTEM,
                       "---- send[%s] synch_mode. prepare see sync",
@@ -2029,13 +2059,6 @@ PlayerAgent::outputDebug()
     {
         M_debug_client.clear();
     }
-
-    /*
-      if ( M_impl->fullstate_.time() == M_impl->current_time_ )
-      {
-      //M_impl->fullstate_.printWithWorld( world() );
-      }
-    */
 }
 
 /*-------------------------------------------------------------------*/
@@ -2398,7 +2421,7 @@ PlayerAgent::doAttentionto( SideID side,
         return false;
     }
 
-    if ( unum == -1 )
+    if ( unum == Unum_Unknown )
     {
         return false;
     }
