@@ -120,6 +120,7 @@ PlayerConfig::setDefaultParam()
     M_debug_write = false;
 
     M_log_dir = "/tmp";
+    M_log_ext = ".log";
 
     M_debug = false;
     M_debug_system = false;
@@ -191,6 +192,7 @@ PlayerConfig::createParamMap( ParamMap & param_map )
         ( "debug_write", "", BoolSwitch( &M_debug_write ) )
 
         ( "log_dir", "", &M_log_dir )
+        ( "log_ext", "", &M_log_ext )
 
         ( "debug", "", BoolSwitch( &M_debug ) )
         ( "debug_system", "", BoolSwitch( &M_debug_system ) )
