@@ -36,7 +36,7 @@
 #include "see_state.h"
 #include "logger.h"
 
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/server_param.h>
 
 namespace rcsc {
 
@@ -378,7 +378,8 @@ SeeState::setViewMode( const ViewWidth & new_width,
                           M_cycles_till_next_see );
             break;
         case ViewWidth::NARROW:
-            std::cerr << "SeeState. TIME_0_00. Narrow is illegal."
+            std::cerr << M_current_time
+                      << " SeeState. TIME_0_00. Narrow is illegal."
                       << std::endl;
             M_synch_type = SYNCH_NO;
             break;
