@@ -99,9 +99,7 @@ private:
     BallObject M_ball; //!< ball object
     // instance of each group
     PlayerCont M_teammates; //!< unum known teammmates
-    PlayerCont M_unknown_teammates; //!< unum unknown teammates
     PlayerCont M_opponents; //!< unum known opponents
-    PlayerCont M_unknown_opponents; //!< unum unknown opponents
     PlayerCont M_unknown_players; //!< unknown players
 
     //////////////////////////////////////////////////
@@ -125,6 +123,8 @@ private:
 
     // updated in  updateOffsideLine();
     double M_offside_line_x; //!< offside line x value
+    int M_offside_line_count; //!< accuracy count for the offside line
+
     double M_defense_line_x; //!< our defense line x value(their offside line)
 
     bool M_exist_kickable_teammate; //!< true if exist kickable teammate
@@ -163,7 +163,12 @@ private:
 
     GameTime M_heard_attack_request_time;
 
+    int M_heard_intercept_unum;
+    int M_heard_intercept_cycle;
+    GameTime M_heard_intercept_time;
+
     //////////////////////////////////////////////////
+
     //! not used
     WorldModel( const WorldModel & );
     //! not used
@@ -335,6 +340,10 @@ public:
 
     void setHeardAttackRequest( const GameTime & current );
 
+    void setHeardInterceptInfo( const int unum,
+                                const int cycle,
+                                const GameTime & current );
+
     /*!
       \brief internal update for action decision
       \param act action effector
@@ -357,84 +366,59 @@ public:
 private:
     /*!
       \brief self localization
-      \param lines seen line info
-      \param markers seen marker flag info
-      \param behind_marker seen but behind marker flag info
+      \param see analyzed see info
       \param current current game time
     */
-    void localizeSelf( const VisualSensor::LineCont & lines,
-                       const VisualSensor::MarkerCont & markers,
-                       const VisualSensor::MarkerCont & behind_markers,
+    void localizeSelf( const VisualSensor & see,
                        const GameTime & current );
 
     /*!
       \brief ball localization
-      \param balls seen ball info
+      \param see analyzed see info
       \param current current game time
     */
-    void localizeBall( const VisualSensor::BallCont & ball,
+    void localizeBall( const VisualSensor & see,
                        const GameTime & current );
 
     /*!
       \brief players localization
-      \param teammates seen teammates that have all identifiers
-      \param unknown_teammates unknown(only side seen) teammates
-      \param opponents seen opponents that have all identifiers
-      \param unknown_opponents unknown(only side seen) opponents
-      \param unknown_players all unknown players
+      \param see analyzed see info
       \param current current game time
     */
-    void localizePlayers( const VisualSensor::PlayerCont & teammates,
-                          const VisualSensor::PlayerCont & unknown_teammates,
-                          const VisualSensor::PlayerCont & opponents,
-                          const VisualSensor::PlayerCont & unknown_opponents,
-                          const VisualSensor::PlayerCont & unknown_players,
-                          const GameTime & current );
+    void localizePlayers( const VisualSensor & see );
 
     /*!
       \brief check player that has team info
       \param player localized info
       \param seen_dist see info data
-      \param old_unum_known previous players that unum is known
-      \param old_unum_known previous players that side is known
+      \param old_known_players old team known players
       \param old_unknown_players previous unknown players
-      \param new_unum_known current players that unum is known
-      \param new_side_known current players that side is known
+      \param new_known_players new team known players
     */
     void checkTeamPlayer( const Localization::PlayerT & player,
                           const double & seen_dist,
-                          PlayerCont & old_unum_known,
-                          PlayerCont & old_side_known,
+                          PlayerCont & old_known_players,
                           PlayerCont & old_unknown_players,
-                          PlayerCont & new_unum_known,
-                          PlayerCont & new_side_known );
+                          PlayerCont & new_known_players );
 
     /*!
       \brief check player that has no identifier. matching to unknown players
       \param player localized info
       \param seen_dist see info data
       \param old_teammates previous seen teammates
-      \param old_unknown_teammates previous seen unknown teammates
       \param old_opponents previous seen opponents
-      \param old_side_opp previous seen unknown opponents
       \param old_unknown_players previous seen unknown player
       \param new_teammates current seen teammates
-      \param new_unknown_teammates current seen unknown teammates
       \param new_opponents current seen opponents
-      \param new_unknown_opponents current seen unknown opponents
       \param new_unknown_players current seen unknown players
     */
     void checkUnknownPlayer( const Localization::PlayerT & player,
                              const double & seen_dist,
                              PlayerCont & old_teammates,
-                             PlayerCont & old_unknown_teammates,
                              PlayerCont & old_opponent,
-                             PlayerCont & old_unknown_opponents,
                              PlayerCont & old_unknown_players,
                              PlayerCont & new_teammates,
-                             PlayerCont & new_unknown_teammates,
                              PlayerCont & new_opponents,
-                             PlayerCont & new_unknown_opponents,
                              PlayerCont & new_unknown_players );
 
     /*!
@@ -599,16 +583,6 @@ public:
           return M_teammates;
       }
 
-    /*!
-      \brief get unknown teammate info
-      \return const reference to the PlayerObject container
-    */
-    const
-    PlayerCont & unknownTeammates() const
-      {
-          return M_unknown_teammates;
-      }
-
     /*!
       \brief get opponent info
       \return const reference to the PlayerObject container
@@ -619,16 +593,6 @@ public:
           return M_opponents;
       }
 
-    /*!
-      \brief get unknown info
-      \return const reference to the PlayerObject container
-    */
-    const
-    PlayerCont & unknownOpponents() const
-      {
-          return M_unknown_opponents;
-      }
-
     /*!
       \brief get unknown player info
       \return const reference to the PlayerObject container
@@ -763,8 +727,8 @@ public:
       }
 
     /*!
-      \brief get opponent nearest to self with confidence count check
-      \param count_thr confidence count threshold
+      \brief get opponent nearest to self with accuracy count check
+      \param count_thr accuracy count threshold
       \return if found, const pointer to the PlayerOjbect. else NULL
     */
     const
@@ -775,6 +739,28 @@ public:
                                  count_thr );
       }
 
+    /*!
+      \brief get the distance to teammate nearest to self wtth accuracy count
+      \param count_thr accuracy count threshold
+      \return distance to the matched opponent. if not found, a big value is returned.
+     */
+    double getDistTeammateNearestToSelf( const int count_thr ) const
+      {
+          const PlayerObject * p = getTeammateNearestToSelf( count_thr );
+          return ( p ? p->distFromSelf() : 65535.0 );
+      }
+
+    /*!
+      \brief get the distance to opponent nearest to self wtth accuracy count
+      \param count_thr accuracy count threshold
+      \return distance to the matched opponent. if not found, a big value is returned.
+     */
+    double getDistOpponentNearestToSelf( const int count_thr ) const
+      {
+          const PlayerObject * p = getOpponentNearestToSelf( count_thr );
+          return ( p ? p->distFromSelf() : 65535.0 );
+      }
+
     /*!
       \brief get teammate nearest to with with confidence count check
       \param count_thr confidence count threshold
@@ -801,6 +787,28 @@ public:
                                  count_thr );
       }
 
+    /*!
+      \brief get the distance to teammate nearest to ball wtth accuracy count
+      \param count_thr accuracy count threshold
+      \return distance to the matched opponent. if not found, a big value is returned.
+     */
+    double getDistTeammateNearestToBall( const int count_thr ) const
+      {
+          const PlayerObject * p = getTeammateNearestToSelf( count_thr );
+          return ( p ? p->distFromBall() : 65535.0 );
+      }
+
+    /*!
+      \brief get the distance to opponent nearest to ball wtth accuracy count
+      \param count_thr accuracy count threshold
+      \return distance to the matched opponent. if not found, a big value is returned.
+     */
+    double getDistOpponentNearestToBall( const int count_thr ) const
+      {
+          const PlayerObject * p = getOpponentNearestToBall( count_thr );
+          return ( p ? p->distFromBall() : 65535.0 );
+      }
+
     // tactical info
 
     /*!
@@ -813,6 +821,15 @@ public:
           return M_offside_line_x;
       }
 
+    /*!
+      \brief get the accuracy count for the offside line
+      \return accuracy count
+     */
+    int offsideLineCount() const
+      {
+          return M_offside_line_count;
+      }
+
     /*!
       \brief get estimated defside line(offside line for opponent) x coodinate
       \return real value
@@ -846,7 +863,7 @@ public:
       \param unum target teammate uniform number
       \return player type Id. if unum is illegal, Default Id is returned.
     */
-    HeteroID getTeammateHeteroID( const int unum ) const
+    HeteroID teammateHeteroID( const int unum ) const
       {
           if ( unum < 1 || 11 < unum )
           {
@@ -854,16 +871,24 @@ public:
                         << unum << std::endl;
               return Hetero_Default;
           }
-          return M_teammate_types[unum-1];
+          return M_teammate_types[ unum - 1 ];
       }
 
+    /*!
+      \brief get player type of the specified teammate
+      \param unum target teammate uniform number
+      \return const pointer to the player type object instance
+     */
+    const
+    PlayerType * teammatePlayerType( const int unum ) const;
+
     /*!
       \brief get player type Id of teammate
       \param unum target teammate uniform number
       \return player type Id. if unum is illegal, Default Id is returned.
     */
     const
-    HeteroID getOpponentHeteroID( const int unum ) const
+    HeteroID opponentHeteroID( const int unum ) const
       {
           if ( unum < 1 || 11 < unum )
           {
@@ -871,9 +896,17 @@ public:
                         << unum << std::endl;
               return Hetero_Unknown;
           }
-          return M_opponent_types[unum-1];
+          return M_opponent_types[ unum - 1 ];
       }
 
+    /*!
+      \brief get player type of the specified opponent
+      \param unum target opponent uniform number
+      \return const pointer to the player type object instance
+     */
+    const
+    PlayerType * opponentPlayerType( const int unum ) const;
+
     // visual memory info
 
     /*!
@@ -881,7 +914,7 @@ public:
       \param angle target direction
       \return confidence count value
     */
-    int getDirCount( const AngleDeg & angle ) const
+    int dirCount( const AngleDeg & angle ) const
       {
           int idx = static_cast< int >( ( angle.degree() - 0.5 + 180.0 )
                                         / DIR_STEP );
@@ -895,33 +928,19 @@ public:
       }
 
     /*!
-      \brief get accuracy count of angle range
-      \param angle base angle
-      \param add_span considerd angle span count
-      \return total accuracy count
+      \brief get max count, sum of count and average count of angle range
+      \param angle center of target angle range
+      \param width angle range
+      \param max_count pointer to variable of max accuracy count
+      \param sum_count pointer to variable of sum of accuracy count
+      \param ave_count pointer to variable of average accuracy count
+      \return steps in the range
     */
-    int getDirRangeCount( const AngleDeg & angle,
-                          const int add_span ) const;
-
-    /*!
-      \brief get max count(lowest confidence) of angle range
-      \param left_angle left start angle
-      \param right_angle right end angle
-      \return lowest accuracy count in range
-    */
-    int getDirRangeMaxCount( const AngleDeg & left_angle,
-                             const AngleDeg & right_angle ) const;
-    /*!
-      \brief get max count(lowest confidence) and average count of angle range
-      \param left_angle left start angle
-      \param right_angle right end angle
-      \param max_count pointer to variable of lowest accuracy
-      \param ave_count pointer to variable of average accuracy
-    */
-    void getDirRangeCount( const AngleDeg & left_angle,
-                           const AngleDeg & right_angle,
-                           int * max_count,
-                           int * ave_count ) const;
+    int dirRangeCount( const AngleDeg & angle,
+                       const double & width,
+                       int * max_count,
+                       int * sum_count,
+                       int * ave_count ) const;
 
     /*!
       \brief get uniform number of heared pass receiver
@@ -1015,7 +1034,8 @@ public:
                 it != end;
                 ++it )
           {
-              if ( (*it)->posCount() > count_thr )
+              if ( (*it)->posCount() > count_thr
+                   || (*it)->isGhost() )
               {
                   continue;
               }
@@ -1036,6 +1056,7 @@ public:
       \param region template parameter. region to be checked
       \param count_thr confdence count threshold for players
       \param with_goalie if true, goalie player is cheked.
+      \return true if some opponent exist
      */
     template < typename REGION >
     bool existOpponentIn( const REGION & region,
@@ -1047,7 +1068,8 @@ public:
                 it != end;
                 ++it )
           {
-              if ( (*it)->posCount() > count_thr )
+              if ( (*it)->posCount() > count_thr
+                   || (*it)->isGhost() )
               {
                   continue;
               }
@@ -1063,6 +1085,72 @@ public:
           return false;
       }
 
+    /*!
+      \brief count the number of teammate exist in the specified region
+      \param region template parameter. region to be checked
+      \param count_thr confdence count threshold for players
+      \param with_goalie if true, goalie player is cheked.
+      \return total count of teammate existed in the region
+     */
+    template < typename REGION >
+    int countTeammatesIn( const REGION & region,
+                          const int count_thr,
+                          const bool with_goalie ) const
+      {
+          int count = 0;
+          const PlayerPtrCont::const_iterator end = teammatesFromSelf().end();
+          for ( PlayerPtrCont::const_iterator it = teammatesFromSelf().begin();
+                it != end;
+                ++it )
+          {
+              if ( (*it)->posCount() > count_thr
+                   || (*it)->isGhost()
+                   || ( (*it)->goalie() && ! with_goalie )
+                   )
+              {
+                  continue;
+              }
+              if ( region.contains( (*it)->pos() ) )
+              {
+                  ++count;
+              }
+          }
+          return count;
+      }
+
+    /*!
+      \brief count the number of opponent exist in the specified region
+      \param region template parameter. region to be checked
+      \param count_thr confdence count threshold for players
+      \param with_goalie if true, goalie player is cheked.
+      \return total count of opponent existed in the region
+     */
+    template < typename REGION >
+    int countOpponentsIn( const REGION & region,
+                          const int count_thr,
+                          const bool with_goalie ) const
+      {
+          int count = 0;
+          const PlayerPtrCont::const_iterator end = opponentsFromSelf().end();
+          for ( PlayerPtrCont::const_iterator it = opponentsFromSelf().begin();
+                it != end;
+                ++it )
+          {
+              if ( (*it)->posCount() > count_thr
+                   || (*it)->isGhost()
+                   || ( (*it)->goalie() && ! with_goalie )
+                   )
+              {
+                  continue;
+              }
+              if ( region.contains( (*it)->pos() ) )
+              {
+                  ++count;
+              }
+          }
+          return count;
+      }
+
 };
 
 }
