@@ -32,9 +32,10 @@
 #ifndef RCSC_RCG_HANDLER_H
 #define RCSC_RCG_HANDLER_H
 
-#include <iostream>
 #include <rcsc/rcg/types.h>
 
+#include <string>
+
 namespace rcsc {
 namespace rcg {
 
@@ -87,7 +88,7 @@ public:
       You can override this.
     */
     virtual
-    int getLogVersion() const
+    int logVersion() const
       {
           return M_log_version;
       }
