@@ -34,7 +34,7 @@
 
 #include <rcsc/trainer/trainer_config.h>
 #include <rcsc/coach/global_world_model.h>
-#include <rcsc/client/basic_client.h>
+#include <rcsc/common/soccer_agent.h>
 #include <rcsc/geom/vector_2d.h>
 #include <rcsc/types.h>
 
@@ -53,7 +53,7 @@ class GlobalVisualSensor;
   \brief basic trainer agent class
 */
 class TrainerAgent
-    : public BasicClient {
+    : public SoccerAgent {
 private:
 
     //! internal implementation object
@@ -104,62 +104,49 @@ public:
     const
     GlobalVisualSensor & visualSensor() const;
 
-    /*!
-      \brief finalize program process
-    */
-    void finalize()
-      {
-          handleExit();
-      }
-
 protected:
     /*!
       \brief analyze command line options
       \param cmd_parser command line parser object
       \return only if "help" option is given, false is returned.
 
-      This method is called from BasicClient::init( argc, argv )
-      BasicClient::init(argc,argv) should be called in main().
-    */
-    virtual
-    bool doInit( CmdLineParser & cmd_parser );
-
-    /*!
-      \brief create connection to the server
-
-      This method is called on the top of BasicClient::run() method.
+      This method is called from SoccerAgent::init( argc, argv )
+      SoccerAgent::init(argc,argv) should be called in main().
       Do NOT call this method by yourself!
     */
     virtual
-    void doConnect();
+    bool initImpl( CmdLineParser & cmd_parser );
 
     /*!
-      \brief send init or reconnect command to server
+      \brief handle start event
+      \return status of start procedure.
 
-      init commad is sent in BasicClient's run() method
-      Do not call this method yourself!
+      This method is called on the top of BasicClient::run() method.
+      The concrete agent must connect to the server and send init command.
+      Do NOT call this method by yourself!
     */
     virtual
-    void sendInitCommand();
-
-    /*!
-      \brief send disconnection command message to server
-      and set the server status to end.
-    */
-    virtual
-    void sendByeCommand();
+    bool handleStart();
 
     /*!
       \brief handle server message event
+
+      Do NOT call this method by yourself!
     */
     virtual
     void handleMessage();
 
     /*!
-      \brief handle select timeout
+      \brief handle timeout event
+      \param timeout_count count of timeout without sensory message.
+      \param waited_msec elapsed milli seconds sinc last sensory message.
+
+      This method is called when select() timeout occurs
+      Do NOT call this method by yourself!
     */
     virtual
-    void handleTimeout();
+    void handleTimeout( const int timeout_count,
+                        const int waited_msec );
 
     /*!
       \brief handle exit event
@@ -167,7 +154,30 @@ protected:
     virtual
     void handleExit();
 
+public:
+    /*!
+      \brief finalize program process
+    */
+    void finalize();
+
 private:
+
+    /*!
+      \brief send init or reconnect command to server
+
+      init commad is sent in BasicClient's run() method
+      Do not call this method yourself!
+    */
+    void sendInitCommand();
+
+    /*!
+      \brief send disconnection command message to server
+      and set the server status to end.
+    */
+    void sendByeCommand();
+
+    ////////////////////////////////////
+
     /*!
       \brief launch each parsing method depending on the message type
       \param msg raw server message
@@ -386,7 +396,7 @@ protected:
       So, do *NOT* call this method by yourself.
     */
     virtual
-    void doAction() = 0;
+    void actionImpl() = 0;
 };
 
 }
